//go:build !(windows && darwin)

package net

import (
	"net"
	"syscall"
)

func SetTcpInfo(conn *net.TCPConn, usingKeepAlive bool, keepAliveIdel, keepAlivePeriod, keepAliveCount, userTimeout int) error {
	rawConn, err := conn.SyscallConn()
	if err != nil {
		return err
	}
	if usingKeepAlive {
		err = rawConn.Control(func(fd uintptr) {
			err = syscall.SetsockoptInt(int(fd), syscall.IPPROTO_TCP, syscall.TCP_KEEPIDLE, keepAliveIdel)
			if err != nil {
				return
			}
			err = syscall.SetsockoptInt(int(fd), syscall.IPPROTO_TCP, syscall.TCP_KEEPINTVL, keepAlivePeriod)
			if err != nil {
				return
			}
			err = syscall.SetsockoptInt(int(fd), syscall.IPPROTO_TCP, syscall.TCP_KEEPCNT, keepAliveCount)
			if err != nil {
				return
			}
		})
	} else {
		err = conn.SetKeepAlive(false)
	}
	if userTimeout > 0 {
		err = rawConn.Control(func(fd uintptr) {
			err = syscall.SetsockoptInt(int(fd), syscall.IPPROTO_TCP, 0x12, userTimeout)
		})
	}
	return err
}
